//
// AUTODESK PROVIDES THIS PROGRAM "AS IS" AND WITH ALL FAULTS. 
// AUTODESK SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTY OF
// MERCHANTABILITY OR FITNESS FOR A PARTICULAR USE.  AUTODESK, INC. 
// DOES NOT WARRANT THAT THE OPERATION OF THE PROGRAM WILL BE
// UNINTERRUPTED OR ERROR FREE.
//
// Use, duplication, or disclosure by the U.S. Government is subject to 
// restrictions set forth in FAR 52.227-19 (Commercial Computer
// Software - Restricted Rights) and DFAR 252.227-7013(c)(1)(ii)
// (Rights in Technical Data and Computer Software), as applicable.
//
//////////////////////////////////////////////////////////////////////////
#include "StdAfx.h"
#include "resource.h"
#include "MyBoxReactor.h"
#include "acadi.h"
#include "dynprops.h"
#include "axboiler.h"
#include "axpnt3d.h"
#include "dbobjptr2.h"

//////////////////////////////////////////////////////////////////////////
void MyBoxReactor::modified(const AcDbObject* obj)
{
  // if we have a n assigned point cloud filter
  if (DocVars.docData().mPCFilter)
  {
    // make sure we have a 3d solid
    AcDb3dSolid *solid = AcDb3dSolid::cast(obj);
    if (solid)
    {
      if (!solid->isErased())
      {
        // extract the minPoint from the existing PCFilter
        AcGePoint3d minPnt, maxPnt;
        DocVars.docData().mPCFilter->mFilterExtents.getMinMaxPoints(minPnt, maxPnt);

        // get the IUknown of the entity, well need it to get the properties for the instance and also so we can set them
        CComPtr<IUnknown> entIUnknown = NULL;
        HRESULT hr = AcAxGetIUnknownOfObject(&entIUnknown, obj->objectId(), acedGetAcadWinApp()->GetIDispatch(TRUE));
        double length=0.001, height=0.001, width=0.001;

        // get the box info
        AcRxClass* pAcrxClass = AcDb3dSolid::desc();
        // Per-instance dynamic property managers
        OPMPerInstancePropertyExtension* pPerInstanceExtension = GET_OPM_PERINSTANCE_EXTENSION_PROTOCOL(pAcrxClass);
        if (NULL != pPerInstanceExtension)
        {
          COleSafeArray sourceNames;
          CComBSTR sourceName;
          // get the property source names stored on the extension object
          pPerInstanceExtension->GetObjectPropertySourceNames((LPVARIANT)sourceNames);
          // find out how many we have
          long start = 0;
          long end = 0;
          sourceNames.GetLBound(1, &start);
          sourceNames.GetUBound(1, &end);
          // loop the property sources
          for (long i=start; i<=end; ++i)
          {
            // extract each one of the property sources out
            CComPtr<IPropertySource> propertySource;
            sourceNames.GetElement(&i, (void*)&sourceName);
            // GetPropertySourceAt does AddRef, so we directly assign pointer here.
            propertySource.p = GET_OPM_PERINSTANCE_PROPERTY_SOURCES()->GetPropertySourceAt(&sourceName);
            // if we have one
            if (propertySource.p)
            {
              // if ok
              if (SUCCEEDED(hr))
              {
                // extract the Dynamic properties from it
                COleSafeArray props;
                hr = propertySource->GetProperties(entIUnknown, props);
                // if ok
                if (SUCCEEDED(hr))
                {
                  // loop all the dynamic properties that we have
                  long start = 0;
                  long end = 0;
                  props.GetLBound(1, &start);
                  props.GetUBound(1, &end);
                  for (long i=start; i<=end; ++i)
                  {
                    // first get the iUknown to make sure we have something
                    CComPtr<IUnknown> unknown = NULL;
                    props.GetElement(&i, (void*)&unknown);
                    // if we have
                    if (unknown.p)
                    {
                      // try and cast it to an updated IDynamicProperty2 
                      CComQIPtr<IDynamicProperty2> dynProp = unknown;
                      // if ok
                      if (dynProp.p)
                      {
                        CComBSTR propName;
                        // extract out the property name
                        dynProp->GetDisplayName(&(propName.m_str));
                        // get the value
                        COleVariant getter;
                        dynProp->GetCurrentValueData(entIUnknown, getter);
                        // find out which property we are looking at
                        if (CString(propName) == _T("Length"))
                          length = ((*(tagVARIANT*)(&getter))).dblVal;
                        if (CString(propName) == _T("Width"))
                          width = ((*(tagVARIANT*)(&getter))).dblVal;
                        if (CString(propName) == _T("Height"))
                          height = ((*(tagVARIANT*)(&getter))).dblVal;;
                      }
                    }
                  }
                }
              }
            }
          }

          // now get the position from the solid
          CComQIPtr<IAcad3DSolid> solidCom(entIUnknown);
          if(solidCom != NULL)
          {
            COleVariant olePosition;
            hr = solidCom->get_Position(&olePosition);
            AcAxPoint3d position = olePosition;

            minPnt.x = position.x - length/2.0;
            minPnt.y = position.y - width/2.0;
            minPnt.z = position.z - height/2.0;
          }

          // finally, update the point cloud
          DocVars.docData().mPCFilter->mFilterExtents.set(minPnt, AcGePoint3d(minPnt.x+length, minPnt.y+width, minPnt.z+height));

          // and touch the point cloud for update
          AcDbBlockTableRecordPointer curSpace(curDoc()->database()->currentSpaceId(), AcDb::kForRead);
          // if ok
          if (curSpace.openStatus() == Acad::eOk)
          {
            AcDbObjectPointer<AcDbEntity> pointCloud( DocVars.docData().mPCFilter->mPointcloudId, AcDb::kForRead);
            // if ok
            if (pointCloud.openStatus() == Acad::eOk)
            {
              AcGsModel *gsModel = curDoc()->database()->gsModel();
              gsModel->onModified(AcGiDrawable::cast(pointCloud.object()), curSpace.object());
            }
          }
        }
      }
    }
  }
}