
// (C) Copyright 2002-2007 by Autodesk, Inc. 
//
// Permission to use, copy, modify, and distribute this software in
// object code form for any purpose and without fee is hereby granted, 
// provided that the above copyright notice appears in all copies and 
// that both that copyright notice and the limited warranty and
// restricted rights notice below appear in all supporting 
// documentation.
//
// AUTODESK PROVIDES THIS PROGRAM "AS IS" AND WITH ALL FAULTS. 
// AUTODESK SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTY OF
// MERCHANTABILITY OR FITNESS FOR A PARTICULAR USE.  AUTODESK, INC. 
// DOES NOT WARRANT THAT THE OPERATION OF THE PROGRAM WILL BE
// UNINTERRUPTED OR ERROR FREE.
//
// Use, duplication, or disclosure by the U.S. Government is subject to 
// restrictions set forth in FAR 52.227-19 (Commercial Computer
// Software - Restricted Rights) and DFAR 252.227-7013(c)(1)(ii)
// (Rights in Technical Data and Computer Software), as applicable.
//
//////////////////////////////////////////////////////////////////////////
#include "StdAfx.h"
#include "resource.h"
#include "MyFilter.h"

//////////////////////////////////////////////////////////////////////////
// this is my point cloud filter implementation, by Fenton Webb, DevTech, Autodesk 27/05/2010
void MyFilter::doFilter(const IAcPcDataBuffer& inBuffer, IAcPcDataBuffer&outBuffer)
{
  AcGePoint3d minPnt, maxPnt;
  mFilterExtents.getMinMaxPoints(minPnt, maxPnt);

  // this is just to show how often the filter function is being called
  acutPrintf(_T("\nfilter is called - min(%.2f,%.2f,%.2f) max(%.2f,%.2f,%.2f)"), minPnt.x, minPnt.y, minPnt.z, maxPnt.x, maxPnt.y, maxPnt.z);

  // ignoring native double buffers for brevity
  if(const_cast<IAcPcDataBuffer&>(inBuffer).nativeDbl())
    return;

  DWORD noOfOutPoints(0);

  AcPcPointFloat *pInPoints = const_cast<IAcPcDataBuffer&>(inBuffer).floatPoints();    
  AcPcPointFloat *pOutPoints = outBuffer.floatPoints();    

  // the points from the point cloud are raw points, we need to convert them into WCS
  // first let's work out the transform matrix
  double x, y, z;    
  inBuffer.offset(x, y, z);
  AcGeMatrix3d mx;
  inBuffer.entityTransform(mx); 

  // now lets loop the points
  int length = inBuffer.size();
  int incrementFactor = 1;
  for (int i=0; i<length; i+=incrementFactor, pInPoints+=incrementFactor)
  {
    // now convert the raw point cloud points into WCS
    AcGePoint3d pt(pInPoints->m_x + x, pInPoints->m_y + y, pInPoints->m_z + z);
    pt = mx*pt;

    // finally check to see if the point lays inside of our box
    if (mFilterExtents.contains(pt))
    {
      // if so, add it to the out array list
      *(pOutPoints+noOfOutPoints) = *pInPoints;
      ++noOfOutPoints;
    }
  }

  // finally, we have less points to show obviously so adjust to suit
  outBuffer.shrink(noOfOutPoints);
}
