// (C) Copyright 2002-2007 by Autodesk, Inc. 
//
// Permission to use, copy, modify, and distribute this software in
// object code form for any purpose and without fee is hereby granted, 
// provided that the above copyright notice appears in all copies and 
// that both that copyright notice and the limited warranty and
// restricted rights notice below appear in all supporting 
// documentation.
//
// AUTODESK PROVIDES THIS PROGRAM "AS IS" AND WITH ALL FAULTS. 
// AUTODESK SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTY OF
// MERCHANTABILITY OR FITNESS FOR A PARTICULAR USE.  AUTODESK, INC. 
// DOES NOT WARRANT THAT THE OPERATION OF THE PROGRAM WILL BE
// UNINTERRUPTED OR ERROR FREE.
//
// Use, duplication, or disclosure by the U.S. Government is subject to 
// restrictions set forth in FAR 52.227-19 (Commercial Computer
// Software - Restricted Rights) and DFAR 252.227-7013(c)(1)(ii)
// (Rights in Technical Data and Computer Software), as applicable.
//

//-----------------------------------------------------------------------------
//----- acrxEntryPoint.cpp
//-----------------------------------------------------------------------------
#include "StdAfx.h"
#include "resource.h"
#include "myFilter.h"
#include "MyBoxReactor.h"

#pragma comment(lib,"AcDbPointCloudObj.lib")
#include "AcPointCloud.h"

//-----------------------------------------------------------------------------
#define szRDS _RXST("asdk")

//-----------------------------------------------------------------------------
//----- ObjectARX EntryPoint
class CAcPointCloudTestApp : public AcRxArxApp 
{
public:
	CAcPointCloudTestApp () : AcRxArxApp () {}

	virtual AcRx::AppRetCode On_kInitAppMsg (void *pkt) {
		// TODO: Load dependencies here

		// You *must* call On_kInitAppMsg here
		AcRx::AppRetCode retCode =AcRxArxApp::On_kInitAppMsg (pkt) ;
		
		// TODO: Add your initialization code here

		return (retCode) ;
	}

	virtual AcRx::AppRetCode On_kUnloadAppMsg (void *pkt) {
		// TODO: Add your code here

		// You *must* call On_kUnloadAppMsg here
		AcRx::AppRetCode retCode =AcRxArxApp::On_kUnloadAppMsg (pkt) ;

		// TODO: Unload dependencies here

		return (retCode) ;
	}

	virtual void RegisterServerComponents () {
	}


	// - asdkAcPointCloudTest.addFilter command (do not rename)
	static void asdkAcPointCloudTestaddFilter(void)
	{
		// pick a point cloud
		ads_name ename;
		ads_point pnt;
		int res = acedEntSel(_T("\nSelect a Point Cloud : "), ename, pnt);
    // if ok
    if (res == RTNORM)
    {
      // then convert the ename to an object id
			AcDbObjectId id;
			acdbGetObjectId(id,ename);

		  // open the selected point cloud entity
		  AcDbEntityPointer ent(id, AcDb::kForWrite);
      // if ok
      if (ent.openStatus() == Acad::eOk)
		  {
		    // reset the point cloud view
        Acad::ErrorStatus es = acdbResetPointCloudDataView(ent);
        // if not ok, possibly not a point cloud
        if (es != Acad::eOk)
        {
          acutPrintf(_T("\nNot a point cloud..."));
          return;
        }

        AcGePoint3d minPnt;
        acedInitGet(RSG_GETZ, NULL);
    		// select the first bounding point
		    res = acedGetPoint(NULL, _T("\nSelect Bottom Left Corner : "), asDblArray(minPnt));
        // if ok
        if (res == RTNORM)
        {
          AcGePoint3d maxPnt;
          acedInitGet(RSG_GETZ, NULL);
          // select the second bounding point
		      res = acedGetPoint(NULL, _T("\nSelect Top Right Corner : "), asDblArray(maxPnt));
          // if ok
          if (res == RTNORM)
          {
            // convert any UCS to world
            AcGeMatrix3d matUcs2Wcs;
            acedGetCurrentUCS(matUcs2Wcs);
            matUcs2Wcs.invert();
            minPnt.transformBy(matUcs2Wcs);
            maxPnt.transformBy(matUcs2Wcs);
            
            // set up and nice extents box
            AcGeBoundBlock3d filterExtents;
            filterExtents.set(minPnt, maxPnt);
            // draw controller box
            DrawSlicerBox(filterExtents);

            // create a new filter object, my one I mean - document specific
            DocVars.docData().mPCFilter = new MyFilter(filterExtents, id);
		        // and apply the filter to the point cloud
		        res = acdbModifyPointCloudDataView(ent.object(), DocVars.docData().mPCFilter);
          }
        }
      }
    }
	}

  static bool DrawSlicerBox(AcGeBoundBlock3d extents)
  {
    // extract the points
    AcGePoint3d minPnt, maxPnt;
    extents.getMinMaxPoints(minPnt, maxPnt);

    // draw a box
    {
      AcDbObjectPointer<AcDb3dSolid> box;
      box.create();
      box->setRecordHistory(true);
      double xDist = fabs(maxPnt.x - minPnt.x) + 0.001;
      double yDist = fabs(maxPnt.y - minPnt.y) + 0.001;
      double zDist = fabs(maxPnt.z - minPnt.z) + 0.001;
      box->createBox(xDist, yDist, zDist);
      box->transformBy(AcGeMatrix3d().setTranslation(AcGeVector3d(minPnt.x + xDist/2, minPnt.y + yDist/2, minPnt.z + zDist/2)));
      AcDbBlockTableRecordPointer curSpace(curDoc()->database()->currentSpaceId(), AcDb::kForWrite);
      // if ok
      if (curSpace.openStatus() == Acad::eOk)
      {
        curSpace->appendAcDbEntity(box.object());
        // add a reactor to it
        box->addReactor(new MyBoxReactor());
        return true;
      }
    }

    return false;
  }

	// - asdkAcPointCloudTest.removeFilter command (do not rename)
	static void asdkAcPointCloudTestremoveFilter(void)
	{
    // pick a point cloud
    ads_name ename;
    ads_point pnt;
    int res = acedEntSel(_T("\nSelect a Point Cloud : "), ename, pnt);
    // if ok
    if (res == RTNORM)
    {
      // then convert the ename to an object id
      AcDbObjectId id;
      acdbGetObjectId(id,ename);
		  // just reset the point cloud view
  		AcDbEntityPointer ent(id, AcDb::kForWrite);
      // if ok
      if (ent.openStatus() == Acad::eOk)
      {
        // reset the point cloud view
        Acad::ErrorStatus es = acdbResetPointCloudDataView(ent);
        // if not ok, possibly not a point cloud
        if (es != Acad::eOk)
        {
          acutPrintf(_T("\nNot a point cloud..."));
          return;
        }
      }
    }
	}
  static void asdkAcPointCloudTesttest(void)
  {
    AcGePoint3d minPnt;
    acedInitGet(RSG_GETZ, NULL);
    // select the first bounding point
    int res = acedGetPoint(NULL, _T("\nSelect Bottom Left Corner : "), asDblArray(minPnt));
    // if ok
    if (res == RTNORM)
    {
      AcGePoint3d maxPnt;
      acedInitGet(RSG_GETZ, NULL);
      // select the second bounding point
      res = acedGetPoint(NULL, _T("\nSelect Top Right Corner : "), asDblArray(maxPnt));
      // if ok
      if (res == RTNORM)
      {
        // set up and nice extents box
        AcGeBoundBlock3d filterExtents;
        filterExtents.set(minPnt, maxPnt);
        DrawSlicerBox(filterExtents);        
          
        while(res == RTNORM)
        {
          AcGePoint3d inside;
          // select the second bounding point
          res = acedGetPoint(NULL, _T("\nPick a point : "), asDblArray(inside));

          if (filterExtents.contains(inside))
            acutPrintf(_T("\nInside"));
        }
      }
    }
  }
} ;

//-----------------------------------------------------------------------------
IMPLEMENT_ARX_ENTRYPOINT(CAcPointCloudTestApp)

ACED_ARXCOMMAND_ENTRY_AUTO(CAcPointCloudTestApp, asdkAcPointCloudTest, addFilter, addFilter, ACRX_CMD_TRANSPARENT, NULL)
ACED_ARXCOMMAND_ENTRY_AUTO(CAcPointCloudTestApp, asdkAcPointCloudTest, removeFilter, removeFilter, ACRX_CMD_TRANSPARENT, NULL)
ACED_ARXCOMMAND_ENTRY_AUTO(CAcPointCloudTestApp, asdkAcPointCloudTest, test, test, ACRX_CMD_TRANSPARENT, NULL)
